################################################################################
##  Copyright (c) 1999-2020 Storix, Inc.
##  San Diego, California, USA
##  All rights reserved
##  This source code is the confidential property of Storix, Inc.
##  and may not be used, copied, or distributed without the written
##  permission of Storix.
################################################################################

# Provides function to migrate software installation to newer version.
# Is sourced by calling command (ie. stinstall).

function chkDevName
{
	# Check the name of the device is not more than 40 characters.
	# If it is, then use the first 12 and last 25 with ... between them.
	local devname=$1
	local count end tmpdev

	count=$($NPRINT $devname | wc -c)
	[ $count -gt 40 ] && {
		# Name is too long. Take first 12 and last 25
		end=$(($count - 24))
		tmpdev=$($NPRINT $devname | cut -c 1-12)
		tmpdev="${tmpdev}...$($NPRINT $devname | cut -c $end-999)"
		devname=$tmpdev
	}

	echo $devname
}

function makeVdev
{
	# Create vdev configuration file in the specified directory.
	# This routine is only used to migrate non-vdev devices and directories
	# to a virtual device. Migrating existing vdevs is handled separately.
	local dev=$1
	local svr=$2
	local type=$3
	local sysbu=$4
	local otherbu=$5
	local outdir=$6
	local admin=$7

	[ $dev = SBDIR -o $dev = SBTAPE ] && {
		# Provided only if system backup disk or type
		shift;shift;shift;shift;shift;shift;shift
		sbdev=$1
		sbtype=$2
		shift;shift
		sbdisks=$*
		sysbu=1
		otherbu=0
	}

	# Check the length of the device name and replace / with ~ for the name
	local name=$(chkDevName $dev | sed "s?/?~?g")

	if [ $admin = 1 ]
	then
		# Creating admin file
		local outfile=$outdir/$svr:$name:info
		[ $dev = SBTAPE ] && outfile=$outdir/$svr:$dev
	else
		# Creating server file
		local outfile=$outdir/vdev:$name
		svr=$LOCAL_HOSTNAME
		[ $license_type = w ] && svr=local
	fi

	# If this file already exists then it was previously created
	# for a BUDIRS directory.
	[ -f $outfile -a $dev != SBTAPE ] && otherbu=1

	echo DVname:$name >$outfile
	echo DVserver:$svr >>$outfile
	echo DVtype:$type >>$outfile
	echo DVlibname: >>$outfile
	echo DVmaxvolsize:0 >>$outfile
	echo DVotherbu:$otherbu >>$outfile
	echo DVstream:s >>$outfile
	echo DVsysbu:$sysbu >>$outfile
	if [ $type = t ]
	then
		if [ $dev = SBTAPE ]
		then
			echo "DVdesc:Local System Backup Tape" >>$outfile
			echo DVdevlist:$sbdev >>$outfile
			echo DVsbtype: >>$outfile
			echo DVsbdisks: >>$outfile
			echo DVlibtype: >>$outfile
		else
			echo "DVdesc:Tape" >>$outfile
			echo DVdevlist:$dev >>$outfile
			echo DVlibtype:n >>$outfile
		fi
	else
		if [ $dev = SBDIR ]
		then
			echo "DVdesc:Local System Backup Disk" >>$outfile
			echo DVdevlist:$sbdev >>$outfile
			echo DVsbtype:$sbtype >>$outfile
			echo DVsbdisks:$sbdisks >>$outfile
			echo DVlibtype: >>$outfile
		else
			echo "DVdesc:Directory" >>$outfile
			echo DVdevlist:$dev >>$outfile
			echo DVlibtype:n >>$outfile
		fi
	fi
}

function migrateFiles 
{
	[ $VRM -lt 720 ] && {
		# Multiple alternate server networks was added in v7.2. Previously
		# user had option of specifying one alternate backup/restore
		# network (ALTHOST1) and one alternate install network (ALTHOST2).
		# Now, all alternate networks are configured as ALTNET in the
		# serverinfo file and there is no limit to the number of networks.

		# Migrate serverinfo file if we are a server and SERVER:info files
		# if we are the controlling admin. We cannot remove ALTHOST1 and
		# ALTHOST2 from serverinfo file here because we need to be able
		# to query the server for these values. These values will be
		# removed the next time the server if saved.
		serverinfolist=
		[ -f $STXPATH/config/serverinfo ] && {
			# We are a server. Migrate this file.
			serverinfolist=$STXPATH/config/serverinfo
		}
		[ $license_type = n -o $license_type = t ] && {
			# Migrate all servers we are the controlling admin of.
			for file in $(ls $STXPATH/*/data/servers/*:info 2>/dev/null)
			do
				serverinfolist="$serverinfolist $file"
			done
		}
		[ ! -z "$serverinfolist" ] && {
			# Migrate all serverinfo files.
			for file in $serverinfolist
			do
				# If ALTNET is in the file then the server record was
				# updated from a v72 admin while the server was still
				# prior to v72.
				[ ! -z "$(grep ^ALTNET $file 2>/dev/null)" ] && continue
				opt=$(grep ^ALTHOST $file | cut -d: -f2 | uniq)
				echo ALTNET:$opt >>$file
			done
		}

		[ $license_type != c ] && {
			oldgrp=$STX_GROUP
			oldgrpid=$STX_GUID
			cat $STXPATH/data/groups 2>/dev/null | while read groupline
			do
				# Migrate administrator files for each group configured
				grp=$(echo $groupline | cut -d: -f1)
				grpid=$(echo $groupline | cut -d: -f3)
				datapath=$STXPATH/$grp/data
				export STX_GROUP=$grp
				export STX_GUID=$grpid

				# Migrate job configuration
				rm -f /tmp/joblist.tmp
				cat $datapath/joblist 2>/dev/null | while read line
				do
					opt=
					oldopt=$(echo "$line" | cut -d: -f11)
					part1=$(echo "$line" | cut -d: -f1-10)
					part2=$(echo "$line" | cut -d: -f12-999)
					[ "$oldopt" = 1 ] && {
						# Get the alternate backup network for server
						server=$(echo "$line" | cut -d: -f2)
						[ -f $datapath/servers/${server}:info ] && {
							# We are the controlling admin and can get the
							# ip/host from this file
							opt=$(grep ^ALTHOST1: \
								$datapath/servers/${server}:info | \
								cut -d: -f2)
						}
						[ -z "$opt" ] && {
							# We are not the controlling admin or did not
							# find it in our file. Must query the server.
							opt=$(strexec $server getserverinfo 2>/dev/null | \
								grep ALTHOST1: | cut -d: -f2)
							[ $? -ne 0 ] && opt=
						}
					}
					newline="${part1}:$opt:${part2}"
					echo "$newline" >>/tmp/joblist.tmp
				done
				mv /tmp/joblist.tmp $datapath/joblist 2>/dev/null

				# Migrate client network boot information
				for file in $(ls $datapath/clients/*:bootinfo 2>/dev/null)
				do
					opt=
					oldopt=$(grep ^INSTALT $file | cut -d" " -f2)
					[ "$oldopt" = 1 ] && {
						# Get the alternate install network for server
						server=$(grep ^BOOTHOST $file | cut -d" " -f2)
						[ -f $datapath/servers/${server}:info ] && {
							# We are the controlling admin and can get the
							# ip/host from this file
							opt=$(grep ^ALTHOST2: \
								$datapath/servers/${server}:info | \
								cut -d: -f2)
						}
						[ -z "$opt" ] && {
							# We are not the controlling admin or did not
							# find it in our file. Must query the server.
							opt=$(strexec $server getserverinfo 2>/dev/null | \
								grep ALTHOST2: | cut -d: -f2)
							[ $? -ne 0 ] && opt=
						}
					}
					grep -v ^INSTALT $file >/tmp/stxinfo.tmp
					[ ! -z "$opt" ] && {
						echo "INSTALTNET $opt" >>/tmp/stxinfo.tmp
					}
					mv /tmp/stxinfo.tmp $file 2>/dev/null
				done

				# Migrate client boot media information
				for file in $(ls $datapath/media/* 2>/dev/null)
				do
					opt=
					oldopt=$(grep ^USEALTHOST $file | cut -d" " -f2)
					[ -z "$oldopt" ] && continue
					[ $oldopt = 1 ] && {
						# Get the alternate install network for server
						server=$(grep ^INSTSERVER $file | cut -d" " -f2)
						[ -f $datapath/servers/${server}:info ] && {
							# We are the controlling admin and can get the
							# ip/host from this file
							opt=$(grep ^ALTHOST2: \
								$datapath/servers/${server}:info | \
								cut -d: -f2)
						}
						[ -z "$opt" ] && {
							# We are not the controlling admin or did not
							# find it in our file. Must query the server.
							opt=$(strexec $server getserverinfo 2>/dev/null | \
								grep ALTHOST2: | cut -d: -f2)
							[ $? -ne 0 ] && opt=
						}
					}
					grep -v ^USEALTHOST $file >/tmp/stxinfo.tmp
					[ ! -z "$opt" ] && {
						echo "INSTALTNET $opt" >>/tmp/stxinfo.tmp
					}
					mv /tmp/stxinfo.tmp $file 2>/dev/null
				done
			done
			export STX_GROUP=$oldgrp
			export STX_GUID=$oldgrpid
		}

		###################################################
		# Migrate config files to new formats and locations
		###################################################
		[ -f $STXPATH/config/serverinfo ] && {
			# This system is currently configured as a server. Migrate
			# serverinfo file to new format.
			# Note that the makeVdev function is assuming that devices in
			# BUDIRS will be created first.
			tmpfile=$STXTEMP/tmpfile
			varlist="ADMIN GROUP ADMINLIST GROUPLIST BOOTDIR ALTNET XIPLIST BUDIRS NETDIRS DEVICELIST"
			echo SVname:$LOCAL_HOSTNAME >$tmpfile
			for var in $varlist
			do
				val=$(grep ^$var: $STXPATH/config/serverinfo | cut -d: -f2)
				case $var in
					ADMIN) newvar=SVadmin;;
					GROUP) newvar=SVgroup;;
					ADMINLIST) newvar=SVadminlist;;
					GROUPLIST) newvar=SVgrouplist;;
					BOOTDIR) newvar=SVbootdir;;
					ALTNET) newvar=SValtnet;;
					XIPLIST) newvar=SVxiplist;;
					BUDIRS) # Create vdev file for each directory
						for dir in $val
						do
							makeVdev $dir x d 0 1 $STXPATH/config 0
						done
						continue;;
					NETDIRS) # Create vdev file for each directory
						for dir in $val
						do
							makeVdev $dir x d 1 0 $STXPATH/config 0
						done
						continue;;
					DEVICELIST) # Create vdev file for each tape device
						devlist=$val
						[ "$devlist" = all ] && {
							# Get list of defined tape devices
							case $SYSTEM in
								aix) 
									devlist=$(lsdev -Cctape 2>/dev/null | \
										cut -d' ' -f1 | grep rmt)
									;;
								linux)
									devlist=$(stdevinfo -t tape 2>/dev/null)
									;;
								solaris)
									devlist=$(stdevinfo -t tape -B 2>/dev/null)
									;;
							esac
							[ -z "$devlist" ] && continue
						}

						for dev in $devlist
						do
							[[ $dev = vdev* ]] && continue
							makeVdev $dev x t 1 1 $STXPATH/config 0
						done
						continue;;
					*) continue;;
				esac
				echo $newvar:$val >>$tmpfile
			done
			mv $tmpfile $STXPATH/config/serverinfo 2>/dev/null
		}

		[ -f $STXPATH/config/io_settings ] && {
			# This system is currently configured as a server. Migrate
			# io_settings to new format.
			tmpfile=$STXTEMP/tmpfile
			varlist="max_readerrs max_syncerrs max_retries pad_buffers"
			>$tmpfile
			for var in $varlist
			do
				val=$(grep ^$var $STXPATH/config/io_settings | cut -d" " -f2)
				case $var in
					max_readerrs) newvar=REmaxreaderrs;;
					max_syncerrs) newvar=REmaxsyncerrs;;
					max_retries) newvar=REmaxretries;;
					pad_buffers) newvar=REpadbuffers;;
					*) continue;;
				esac
				echo $newvar:$val >>$tmpfile
			done
			mv $tmpfile $STXPATH/config/io_settings 2>/dev/null
		}

		[ -f $STXPATH/config/virtual_devices ] && {
			# This system is currently configured as a server. Migrate
			# virtual devices settings to new format. Note that we do not
			# set DVname because we do not know the server named used to
			# configure this system as a server.
			cat $STXPATH/config/virtual_devices | while read line
			do
				[ -z "$line" ] && continue
				var=$(echo $line | cut -d: -f1)		# device
				outfile=$STXPATH/config/vdev:$var
				echo DVname:$var >>$outfile
				var=$(echo $line | cut -d: -f2)		# libopt
				echo DVlibtype:$var >>$outfile
				var=$(echo $line | cut -d: -f3)		# type
				echo DVstream:$var >>$outfile
				var=$(echo "$line" | cut -d: -f4)		# desc
				echo "DVdesc:$var" >>$outfile

				# Remove /dev from each device name
				var=$(echo $line | cut -d: -f5)		# devlist
				devlist=
				for dev in $var
				do
					[ -z "$devlist" ] && {
						devlist="$(echo $dev | sed "s%^/dev/%%")"
						continue
					}
					devlist="$devlist $(echo $dev | sed "s%^/dev/%%")"
				done
				echo DVdevlist:$devlist >>$outfile

				var=$(echo $line | cut -d: -f6)		# libname
				echo DVlibname:$var >>$outfile

				# New options only found in v72
				echo DVtype:t >>$outfile
				echo DVmaxvolsize:0 >>$outfile
				echo DVsysbu:1 >>$outfile
				echo DVotherbu:1 >>$outfile
			done
			rm -f $STXPATH/config/virtual_devices
		}

		[ -f $STXPATH/config/sbdiskconfig ] && {
			# Migrate client local system backup disk configuration file
			sbdisks=$(cut -d: -f1 $STXPATH/config/sbdiskconfig)	# disks
			sbdev=$(cut -d: -f2 $STXPATH/config/sbdiskconfig)	# mountpoint
			sbtype=$(cut -d: -f3 $STXPATH/config/sbdiskconfig)	# type
			makeVdev SBDIR x d 1 0 $STXPATH/config 0 $sbdev $sbtype $sbdisks
			rm -f $STXPATH/config/sbdiskconfig
		}

		[ -f $STXPATH/config/tsm/tsmclient:info ] && {
			outfile=$STXTEMP/tsmclientinfo
			echo TCname:$LOCAL_HOSTNAME >$outfile
			cat $STXPATH/config/tsm/tsmclient:info | while read line
			do
				var=$(echo $line | cut -d" " -f1)
				val=$(echo $line | cut -d" " -f2)
				case $var in
					password) newvar=TCpassword;;
					nodename) newvar=TCnodename;;
					*) newvar=$var;;
				esac
				echo $newvar:$val >>$outfile
			done
			mv $outfile $STXPATH/config/tsm/tsmclient:info
		}

		[ $license_type = c ] && {
			# Create a SBTAPE device for the first device found on this client.
			# We do this even if the client is not using the SBTAPE option
			# for a current job because the client could be updated after the
			# admin is updated.
			local devlist
			case $SYSTEM in
				aix) devlist=$(lsdev -Cctape 2>/dev/null | \
						cut -d' ' -f1 | grep rmt);;
				linux) devlist=$(stdevinfo -t tape 2>/dev/null);;
				solaris) devlist=$(stdevinfo -t tape -B 2>/dev/null);;
			esac
			for dev in $devlist
			do
				makeVdev SBTAPE x t 1 0 $STXPATH/config 0 $dev x
				break
			done
		}

		[ $license_type != c ] && {
			# Administrator migration
			[ $license_type != w ] && {
				# Get admin ip address
				adminip=$(sthost $(hostname) | cut -d" " -f2)
			}
			tmpfile=$STXTEMP/tmpfile

			# Migrate config files for each group configured
			# Note that we will migrate client configuration even if we
			# are workstation in case user has switched or would like to
			# switch between license types.
			oldgrp=$STX_GROUP
			oldgrpid=$STX_GUID
			cat $STXPATH/data/groups 2>/dev/null | while read groupline
			do
				# Migrate administrator files for each group configured
				grp=$(echo $groupline | cut -d: -f1)
				grpid=$(echo $groupline | cut -d: -f3)
				datapath=$STXPATH/$grp/data
				configpath=$STXPATH/$grp/config
				export STX_GROUP=$grp
				export STX_GUID=$grpid

				echo "  Migrating data for group \"$grp\" ..."

				# Save previous versions in case we need to reference them
				savedir=$STXPATH/v${VRM}/$grp
				mkdir -p $savedir 2>/dev/null

				##################
				# Clients files
				##################
				clpath=$datapath/clients
				[ $license_type = n -o $license_type = t ] && {
					# Migrate from clientlist to client:info
					echo "    Migrating client configuration ..."

					[ -s $datapath/clientlist ] && {
						while read line
						do
							name=$(echo $line | cut -d: -f1)	# name
							outfile=$clpath/$name:info
							echo CLname:$name >$outfile
							opt=$(echo $line | cut -d: -f2)		# encryption
							[ "$opt" != 0 -a "$opt" != 1 ] && opt=0
							echo CLenc:$opt >>$outfile
							opt=$(echo $line | cut -d: -f5)		# tsm
							[ "$opt" != 0 -a "$opt" != 1 ] && opt=0
							echo CLtsm:$opt >>$outfile
							echo CLsmb:0 >>$outfile
							echo CLora:0 >>$outfile
							echo CLsparse:Y >>$outfile
							echo CLatime:0 >>$outfile
						done < $datapath/clientlist
					}
					mv $datapath/clientlist $savedir 2>/dev/null

					##################
					# Client network boot bootinfo files
					##################
					for file in $(ls $clpath/*:bootinfo 2>/dev/null)
					do
						rm -f $tmpfile
						varlist="CLIENTNAME BOOTHOST BOOTSERVERIP BOOTNAME BOOTGATE BOOTMASK HWADDR PROMPTMODE INSTHOST INSTALTNET BOOTALTNET BOOTCLIENTIP DISABLE SYSTEM PLATFORM KERNEL CONSOLE TERM VGOPT RESTDEV BUSEQ APISERVER APIfromclient APIBACKUPID SERVERIP GATEWAY SUBMASK NETDEV CLIENTIP RINGSPEED ENTIF ENTCON RIMSTART RIMLOGIN RIMDEV RIMIP RIMMASK RIMSPEED RIMENTIF RIMENTCON RIMPWD"
						for var in $varlist
						do
							val=$(grep ^$var $file | cut -d" " -f2-99)
							[ ! -z "$val" -a $var = "$val" ] && {
								# Probably a line that does not have a value
								[ $(grep ^$var $file | wc -w) -eq 1 ] && val=
							}
							case $var in
								CLIENTNAME) newvar=NBclient;;
								BOOTHOST) newvar=NBboothost;;
								BOOTSERVERIP) newvar=NBbootserverip;;
								BOOTNAME) newvar=NBbootname;;
								BOOTGATE) newvar=NBbootgate;;
								BOOTMASK) newvar=NBbootmask;;
								HWADDR) newvar=NBhwaddr;;
								PROMPTMODE) newvar=NBprompt;;
								INSTHOST) newvar=NBinsthost;;
								INSTALTNET) newvar=NBinstaltnet;;
								BOOTALTNET) newvar=NBbootaltnet;;
								BOOTCLIENTIP) newvar=NBbootclientip;;
								DISABLE) newvar=NBdisable;;
								SYSTEM) newvar=NBsystem;;
								PLATFORM) newvar=NBplatform;;
								KERNEL) newvar=NBkernel;;
								CONSOLE) newvar=NBconsole;;
								TERM) newvar=NBterm;;
								VGOPT) newvar=NBvgopt;;
								RESTDEV) newvar=NBrestdev
									if [[ $val = /* ]]
									then
										basefile=$(basename $val)
										val=$(dirname $val)
										[[ $val = /images ]] && val=SBDIR
										echo NBbasefile:$basefile >>$tmpfile
									else
										echo NBbasefile: >>$tmpfile
									fi
									;;
								BUSEQ) newvar=NBbuseq;;
								APISERVER) newvar=NBapiserver;;
								APIfromclient) newvar=NBfromclient;;
								APIBACKUPID) newvar=NBapibackupid;;
								SERVERIP) newvar=NBserverip;;
								GATEWAY) newvar=NBgateway;;
								SUBMASK) newvar=NBsubmask;;
								NETDEV) newvar=NBnetdev;;
								CLIENTIP) newvar=NBclientip;;
								RINGSPEED) newvar=NBringspeed;;
								ENTIF) newvar=NBentif;;
								ENTCON) newvar=NBentcon;;
								RIMSTART) newvar=NBrimstart;;
								RIMLOGIN) newvar=NBrimlogin;;
								RIMDEV) newvar=NBrimdev;;
								RIMIP) newvar=NBrimip;;
								RIMMASK) newvar=NBrimmask;;
								RIMSPEED) newvar=NBrimspeed;;
								RIMENTIF) newvar=NBrimentif;;
								RIMENTCON) newvar=NBrimentcon;;
								RIMPWD) newvar=NBrimpwd;;
								*) continue;;
							esac
							echo $newvar:$val >>$tmpfile
						done
						echo NBarch: >>$tmpfile
						cp $file $savedir 2>/dev/null
						mv $tmpfile $file 2>/dev/null
					done
				}

				##################
				# Client sparse setting
				##################
				for file in $(ls $clpath/*:backend 2>/dev/null)
				do
					# Only migrating sparse setting in the file. All other
					# settings were disabled or moved elsewhere at v6.1
					val=$(grep ^sparse $file | cut -d" " -f2)
					[ -z "$val" ] && {
						# Backward compatibility (AIX only)
						val=$(grep ^backend $file | cut -d" " -f2) 
					}
					[ "$val" != T -a "$val" != N ] && {
						mv $file $savedir 2>/dev/null
						continue
					}

					namelist=$(basename $file | cut -d: -f1)
					[ $namelist = all ] && {
						# Change for all clients
						namelist=
						for clfile in $(ls $clpath/*:info 2>/dev/null)
						do
							name=$(basename $clfile | cut -d: -f1)
							namelist="$namelist $name"
						done
					}

					for name in $namelist
					do
						grep -v ^CLsparse: $clpath/$name:info >$tmpfile
						echo CLsparse:N >>$tmpfile
						mv $tmpfile $clpath/$name:info
					done
					mv $file $savedir 2>/dev/null
				done

				##################
				# Client local system backup disk
				##################
				for file in $(ls $clpath/*:sbdiskconfig 2>/dev/null)
				do
					name=$(basename $file | cut -d: -f1)
					outfile=$clpath/$name:SBDIR
					echo DVserver:$name >$outfile
					opt=$(cut -d: -f1 $file)	# disks
					echo DVsbdisks:$opt >>$outfile
					opt=$(cut -d: -f2 $file)	# mountpoint
					echo DVdevlist:$opt >>$outfile
					opt=$(cut -d: -f3 $file)	# type
					echo DVsbtype:$opt >>$outfile
					echo "DVdesc:Local System Backup Disk" >>$outfile
					echo DVlibtype: >>$outfile
					echo DVlibname: >>$outfile
					echo DVmaxvolsize:0 >>$outfile
					echo DVname:SBDIR >>$outfile
					echo DVotherbu:0 >>$outfile
					echo DVstream:s >>$outfile
					echo DVsysbu:1 >>$outfile
					echo DVtype:d >>$outfile
					mv $file $savedir 2>/dev/null

					# Copy the file to the client and ignore any errors
					# because the client might be down. We want to copy even
					# though the client may have already created the file
					# because we have found that STXPATH/config/sbdiskconfig
					# was not created in migration from 6.3 to 7.1
					islocal=1
					[ $license_type != w ] && {
						clip=$(sthost $name 2>/dev/null | cut -d" " -f2)
						[ "$clip" != "$adminip" ] && islocal=0
					}

					# Copy file to client
					if [ $islocal = 1 ]
					then
						# We are the client
						cp $outfile $STXPATH/config/vdev:SBDIR 2>/dev/null
					else
						cat $outfile 2>/dev/null | \
							strexec -i $name copyfile SBDIR 2>/dev/null
					fi
				done

				##################
				# Client snapshot settings
				##################
				for file in $(ls ${clpath}/*:snsettings 2>/dev/null)
				do
					name=$(basename $file | cut -d: -f1)
					echo SNclient:$name >$tmpfile
					varlist="DEVLIST ACCESS ERROPT CHUNKSIZE SNAPSHOT"
					for var in $varlist
					do
						val=$(grep ^$var $file | cut -d: -f2-99)
						case $var in
							DEVLIST) newvar=SNdevlist;;
							ACCESS) newvar=SNaccess;;
							ERROPT) newvar=SNerropt;;
							CHUNKSIZE) newvar=SNchunksize;;
							SNAPSHOT) newvar=SNsnapshot;;
							*) continue;;
						esac
						echo $newvar:$val >>$tmpfile
					done
					cp $file $savedir 2>/dev/null
					mv $tmpfile $file 2>/dev/null
				done

				##################
				# Servers
				##################
				# Create the new configuration file for all servers
				# even if we are not the controlling admin.
				# Note that the makeVdev function is assuming that
				# devices in BUDIRS will be created first.
				svpath=${datapath}/servers
				local nonadminsvrs=
				[ $license_type != w ] && {
					echo "    Migrating server and device configuration ..."
				}
				cat $datapath/serverlist 2>/dev/null | while read name
				do

					# See if we run commands remotely. We dont want to use
					# strexec if we are the server because strexecd may not
					# have restarted yet.
					remopt=
					nameip=$(sthost $name 2>/dev/null | cut -d" " -f2)
					[ "$nameip" != "$adminip" ] && remopt="strexec $name "

					file=$svpath/$name:info
					[ ! -f $file ] && {
						# We are not the controlling admin so need to get the
						# serverinfo from the server. Note that the server
						# must have already been migrated to v72.
						nonadminsvrs="${nonadminsvrs}$name "
						rm -f $tmpfile
						strexec $name getserverinfo | \
								egrep -v "IS_ADMIN|IS_GROUP" >$tmpfile 2>&1 || {
							$PRINT "\07  Could not migrate server $name."
							cat $tmpfile 2>/dev/null
							$PRINT "    You must reconfigure this server manually once the problem\n    has been resolved."
							rm -f $tmpfile
							sleep 5
							continue
						}
						[ -z "$(grep ^SValtnet: $tmpfile 2>/dev/null)" ] && {
							$PRINT "\07  Could not migrate server $name because it has not been updated to\n    SBAdmin v7.2. You must reconfigure this server manually once it\n    has been updated."
							rm -f $tmpfile
							sleep 5
							continue
						}
						[ -s $tmpfile ] && {
							echo SVname:$name >$svpath/$name:info
							egrep -v "^IS_ADMIN|^IS_GROUP" \
								$tmpfile >>$svpath/$name:info
						}
						rm -f $tmpfile
						continue
					}

					# Go through values and create the config file
					echo SVname:$name >$tmpfile
					varlist="ADMIN GROUP ADMINLIST GROUPLIST BOOTDIR ALTNET XIPLIST BUDIRS NETDIRS DEVICELIST"
					for var in $varlist
					do
						val=$(grep ^$var: $file | cut -d: -f2)
						case $var in
							ADMIN) newvar=SVadmin;;
							GROUP) newvar=SVgroup;;
							ADMINLIST) newvar=SVadminlist;;
							GROUPLIST) newvar=SVgrouplist;;
							BOOTDIR) newvar=SVbootdir;;
							ALTNET) newvar=SValtnet;;
							XIPLIST) newvar=SVxiplist;;
							BUDIRS) # Create vdev file for each directory
								for dir in $val
								do
									outdir=$datapath/devices
									makeVdev $dir $name d 0 1 $outdir 1
								done
								continue;;
							NETDIRS) # Create vdev file for each directory
								for dir in $val
								do
									outdir=$datapath/devices
									makeVdev $dir $name d 1 0 $outdir 1
								done
								continue;;
							DEVICELIST) # Create vdev file for each tape device
								[ "$val" = all ] && {
									# Get list of tape virtual devices already
									# configured on the server. These commands
									# will only work if server has been updated
									# to 72.
									devlist=$(${remopt}stremcmd lsdevs \
										-tape 1 2>/dev/null)
									[ $? -ne 0 -o -z "$devlist" ] && continue
									for vdev in $devlist
									do
									  opts=$(${remopt}stremcmd getvdevopts \
										$vdev 2>/dev/null)
									  [ $? -ne 0 -o -z "$opts" ] && continue
									  vdev=$(echo $vdev |sed "s?/?~?g")	
									  outfile=$datapath/devices/$name:$vdev:info
									  echo "$opts" >$outfile
									done
									continue
								}

								for dev in $val
								do
									[[ $dev = vdev* ]] && continue
									outdir=$datapath/devices
									makeVdev $dev $name t 1 1 $outdir 1
								done
								continue;;
							*) continue;;
						esac
						echo $newvar:$val >>$tmpfile
					done
					cp $file $savedir 2>/dev/null
					mv $tmpfile $file 2>/dev/null
				done
				mv $datapath/serverlist $savedir 2>/dev/null

				[ $license_type = w ] && {
					##################
					# Tape and Disk Devices (workstation only, non-vdev)
					##################
					echo "    Migrating device configuration ..."
					for var in BUDIRS DEVICELIST
					do
						val=$(grep ^$var $svpath/local:info | cut -d: -f2)
						[ -z "$val" ] && continue

						case $var in
							BUDIRS) # Create vdev file for each directory
								for dir in $val
								do
									outdir=$datapath/devices
									makeVdev $dir local d 0 1 $outdir 1
									vdev=$(chkDevName $dir | sed "s?/?~?g")
									cp $outdir/local:$vdev:info \
										$STXPATH/config/vdev:$vdev 2>/dev/null
								done
								;;
							DEVICELIST) # Create vdev file for each tape device
								devlist=$val
								[ $val = all ] && {
									# Get list of tape devices
									case $SYSTEM in
										aix) 
											devlist=$(lsdev -Cctape \
												2>/dev/null | cut -d' ' -f1 | \
												grep rmt)
											;;
										linux)
											devlist=$(stdevinfo -t tape \
												2>/dev/null)
											;;
										solaris)
											devlist=$(stdevinfo -t tape -B \
												2>/dev/null)
											;;
									esac
								}

								for dev in $devlist
								do
									[[ $dev = vdev* ]] && continue
									outdir=$datapath/devices
									makeVdev $dev local t 1 1 $outdir 1
									vdev=$(chkDevName $dev | sed "s?/?~?g")
									cp $outdir/local:$vdev:info \
										$STXPATH/config/vdev:$vdev 2>/dev/null
								done
						esac
					done
					mv $svpath/local:info $savedir 2>/dev/null
				}

				##################
				# Virtual devices
				##################
				for file in $(ls $svpath/*:virdevs 2>/dev/null)
				do
					name=$(basename $file | cut -d: -f1)
					cat $file | while read line
					do
						[ -z "$line" ] && continue
						opt=$(echo $line | cut -d: -f1)		# device
						outfile=$datapath/devices/$name:$opt:info
						echo DVserver:$name >$outfile
						echo DVname:$opt >>$outfile
						opt=$(echo $line | cut -d: -f2)		# libopt
						echo DVlibtype:$opt >>$outfile
						opt=$(echo $line | cut -d: -f3)		# type
						echo DVstream:$opt >>$outfile
						opt=$(echo "$line" | cut -d: -f4)		# desc
						echo "DVdesc:$opt" >>$outfile

						# Remove /dev from each device name
						opt=$(echo $line | cut -d: -f5)		# devlist
						devlist=
						for dev in $opt
						do
							[ -z "$devlist" ] && {
								devlist="$(echo $dev | sed "s%^/dev/%%")"
								continue
							}
							devlist="$devlist $(echo $dev | sed "s%^/dev/%%")"
						done
						echo DVdevlist:$devlist >>$outfile

						opt=$(echo $line | cut -d: -f6)		# libname
						echo DVlibname:$opt >>$outfile

						# New options only found in v72
						echo DVtype:t >>$outfile
						echo DVmaxvolsize:0 >>$outfile
						echo DVsysbu:1 >>$outfile
						echo DVotherbu:1 >>$outfile
					done
					mv $file $savedir 2>/dev/null
				done

				##################
				# Server read-error handling
				##################
				for file in $(ls $svpath/*:readerrs 2>/dev/null)
				do
					name=$(basename $file | cut -d: -f1)
					echo REserver:$name >$tmpfile
					varlist="max_readerrs max_syncerrs max_retries pad_buffers"
					for var in $varlist
					do
						val=$(grep ^$var $file | cut -d" " -f2)
						case $var in
							max_readerrs) newvar=REmaxreaderrs;;
							max_syncerrs) newvar=REmaxsyncerrs;;
							max_retries) newvar=REmaxretries;;
							pad_buffers) newvar=REpadbuffers;;
							*) continue;;
						esac
						echo $newvar:$val >>$tmpfile
					done
					cp $file $savedir 2>/dev/null
					mv $tmpfile $file 2>/dev/null
				done

				##################
				# Random libraries
				##################
				[ -s $configpath/library_info ] && {
					liblines=$(grep ^LIB: $configpath/library_info)
					for line in $liblines
					do
						name=$(echo $line | cut -d: -f2)

						# Migrate library definition
						outfile=$datapath/libraries/$name:info
						echo LBname:$name >$outfile
						opt=$(echo $line | cut -d: -f3)		# libdev
						echo LBdevice:$opt >>$outfile
						opt=$(echo $line | cut -d: -f4)		# numdrives
						echo LBnumdrives:$opt >>$outfile
						opt=$(echo $line | cut -d: -f5)		# numtapes
						echo LBnumtapes:$opt >>$outfile
						opt=$(echo $line | cut -d: -f6)		# end action
						echo LBlastaction:$opt >>$outfile
						opt=$(grep ^CMD:$name:D2S: $configpath/library_info | \
							cut -d: -f4)
						echo LBcmdD2S:$opt >>$outfile
						opt=$(grep ^CMD:$name:S2D: $configpath/library_info | \
							cut -d: -f4)
						echo LBcmdS2D:$opt >>$outfile
						opt=$(grep ^CMD:$name:INV: $configpath/library_info | \
							cut -d: -f4)
						echo LBcmdinv:$opt >>$outfile

						# Migrate slot information for library
						# File must contain all possible slots even if they
						# are not currently used (4 drives, 20 tapes).
						outfile=$datapath/libraries/$name:slots
						>$outfile
						for d in 1 2 3 4
						do
							opt=$(grep ^DRIVE:$name:$d: \
								$configpath/library_info | cut -d: -f4)
							echo LSdrive$d:$opt >>$outfile
						done

						t=1
						while [ $t -le 80 ]
						do
							line=$(grep ^TAPE:$name:[0-9]:$t: \
								$configpath/library_info)
							if [ -z "$line" ]
							then
								echo LStape$t: >>$outfile
							else
								opt=$(echo $line | cut -d: -f5)
								echo LStape$t:$opt >>$outfile
							fi
							t=$((t + 1))
						done
					done
				}
				mv $configpath/library_info $savedir 2>/dev/null

				##################
				# TSM configuration
				##################
				# TSM admin configuration files
				for file in $(ls $datapath/tsm/tsmadmin:* 2>/dev/null)
				do
					[ -z "$(grep ^adminuser $file 2>/dev/null)" ] && continue
					name=$(basename $file | cut -d: -f2-99)
					outfile=$datapath/tsm/$name:admin
					echo TAserver:$name >$outfile
					opt=$(grep ^adminuser $file | cut -d" " -f2)
					echo TAadminuser:$opt >>$outfile
					opt=$(grep ^adminpass $file | cut -d" " -f2)
					echo TAadminpass:$opt >>$outfile
					mv $file $savedir 2>/dev/null
				done

				##################
				# TSM server configuration files
				##################
				for file in $(ls $datapath/tsm/tsmserver:* 2>/dev/null)
				do
					[ -z "$(grep SErvername $file 2>/dev/null)" ] && continue
					name=$(basename $file | cut -d: -f2-99)
					outfile=$datapath/tsm/$name:server
					>$outfile
					cat $file | while read line
					do
						var=$(echo $line | cut -d" " -f1)
						val=$(echo $line | cut -d" " -f2)
						case $var in
							SErvername) newvar=TSname;;
							COMPRESSIon) newvar=TScompression;;
							TCPPort) newvar=TStcpport;;
							TCPServeraddress) newvar=TStcpserveraddress;;
							TCPBuffsize) newvar=TStcpbuffsize;;
							TCPWindowsize) newvar=TStcpwindowsize;;
							TCPNodelay) newvar=TStcpnodelay;;
							PASSWORDAccess) newvar=TSpasswordaccess;;
							COMMMethod) newvar=TScommmethod;;
							*) newvar=$var;;
						esac
						echo $newvar:$val >>$outfile
					done
					mv $file $savedir 2>/dev/null
				done

				##################
				# TSM client configuration files
				##################
				for file in $(ls $datapath/tsm/tsmclient:* 2>/dev/null)
				do
					[ -z "$(grep ^nodename $file 2>/dev/null)" ] && continue
					name=$(basename $file | cut -d: -f2-99)
					outfile=$datapath/tsm/$name:client
					echo TCname:$name >$outfile
					cat $file | while read line
					do
						var=$(echo $line | cut -d" " -f1)
						val=$(echo $line | cut -d" " -f2)
						case $var in
							password) newvar=TCpassword;;
							nodename) newvar=TCnodename;;
							*) newvar=$var;;
						esac
						echo $newvar:$val >>$outfile
					done
					mv $file $savedir 2>/dev/null
				done

				##################
				# Backup profiles (default and custom)
				##################
				echo "    Migrating backup profile configuration ..."
				profdir=$datapath/profiles
				for type in prof job
				do
					for file in $(ls $profdir/${type}.* 2>/dev/null)
					do
						if [ $type = prof ]
						then
							name=$(basename $file | cut -d. -f2)
							outfile=$datapath/profiles/$name:info
						else
							name=$(basename $file | sed -e "s/^job.//" \
								-e "s/.profile$//")
							outfile=$datapath/jobs/$name:profile
						fi
						>$outfile
						varlist="bufsize butype compress datalist desc eject hostperm level profname orascript orastart orastop postprog postsnapprog ppmap preprog presnapprog printlab printqueue priority rawdev rawpp retaindays retainnum rewind svpostprog svpreprog"
						for var in $varlist
						do
							val=$(grep ^$var: $file | cut -d: -f2-99)
							case $var in
								profname) var=name ;;
								eject|orastart|orastop|ppmap|printlab|\
										rawdev|rawpp|rewind)
									# Boolean values converted from y/n to 1/0
									[ "$val" = y ] && val=1 || val=0 ;;
							esac
							[ $var = profname ] && var=name
							echo "PF$var:$val" >>$outfile
						done
						mv $file $savedir 2>/dev/null
					done
				done

				##################
				# Backup jobs files
				##################
				[ -s $datapath/joblist ] && {
					echo "    Migrating backup job configuration ..."
					jbpath=$datapath/jobs
					while read line
					do
						local svr dev is_sbdev sbtape
						local islocal clip is_sysbu
						is_sbdev=0
						is_sysbu=0
						sbtape=

						name=$(echo $line | cut -d: -f1)	# jobid
						outfile=$jbpath/$name:info
						echo JBjobid:$name >$outfile
						svr=$(echo $line | cut -d: -f2) 	# server
						echo JBserver:$svr >>$outfile
						opt=$(echo $line | cut -d: -f3) 	# when
						echo JBwhen:$opt >>$outfile
						opt=$(echo $line | cut -d: -f4) 	# profile
						[ $(grep ^PFbutype: $datapath/profiles/$opt:info | \
								cut -d: -f2) = 1 ] && {
							is_sysbu=1
							[ $svr = local ] && is_sbdev=1
						}
						echo JBprofile:$opt >>$outfile
						dev=$(echo $line | cut -d: -f5) 	# devname
						[ $is_sbdev = 1 ] && {
							if [[ $dev = /* ]]
							then dev=SBDIR
							else
								[ $license_type != w ] && {
									sbtape=$dev
									dev=SBTAPE
								}
							fi
						}
						vdev=$(chkDevName $dev | sed "s?/?~?g")
						echo JBdevname:$vdev >>$outfile
						opt=$(echo $line | cut -d: -f6) 	# clients
						islocal=1
						[ $license_type != w ] && {
							clip=$(sthost $opt 2>/dev/null | cut -d" " -f2)
							[ "$clip" != "$adminip" ] && islocal=0
						}
						[ $dev = SBTAPE ] && {
							# Create virtual device record for this device
							# and try to copy it to the client. Ignore if
							# the copy fails because the client may not be up.
							outdir=$datapath/clients
							makeVdev SBTAPE $opt t 1 0 $outdir 1 $sbtape x

							# Copy file to client
							if [ $islocal = 1 ]
							then
								# We are the client
								cp $outdir/$opt:SBTAPE \
									$STXPATH/config/vdev:SBTAPE 2>/dev/null
							else
								cat $outdir/$opt:SBTAPE 2>/dev/null | \
									strexec -i $opt copyfile SBTAPE 2>/dev/null
							fi
						}
						[ $dev != SBDIR -a $dev != SBTAPE -a -z \
								"$(echo $nonadminsvrs | grep -w $svr)" ] && {
							# We are the controlling admin. Make sure the
							# device has been created for this server. The
							# only way the device has not already been created
							# is if the device is non-vdev tape and DEVICELIST
							# was set to "all" and the server is either not
							# available or the tape device is not currently
							# configured.
							outdir=$datapath/devices
							[ ! -f $outdir/$svr:$vdev:info ] && {
								if [[ $dev = /* ]]
								then
									if [ $is_sysbu = 1 ]
									then
										makeVdev $dev $svr d 1 0 $outdir 1
									else
										makeVdev $dev $svr d 0 1 $outdir 1
									fi
								else
									makeVdev $dev $svr t 1 1 $outdir 1
								fi
							}

							# Try to copy the file to the server
							islocal=1
							[ $license_type != w ] && {
								svip=$(sthost $svr 2>/dev/null | cut -d" " -f2)
								[ "$svip" != "$adminip" ] && islocal=0
							}
							if [ $islocal = 1 ]
							then
								cp $outdir/$svr:$vdev:info \
									$STXPATH/config/vdev:$vdev 2>/dev/null
							else
								cat $outdir/$svr:$vdev:info 2>/dev/null | \
									strexec -i $svr copyfile \
									VDEV:$vdev 2>/dev/null
							fi
						}
						echo JBclients:$opt >>$outfile
						opt=$(echo "$line" | cut -d: -f7) 	# cronlist
						echo "JBcronlist:$opt" >>$outfile
						opt=$(echo $line | cut -d: -f8) 	# datalist
						echo JBdatalist:$opt >>$outfile
						opt=$(echo "$line" | cut -d: -f9) 	# userdesc
						echo "JBdesc:$opt" >>$outfile
						opt=$(echo $line | cut -d: -f10) 	# delopt
						echo JBdelopt:$opt >>$outfile
						opt=$(echo $line | cut -d: -f11) 	# altnet
						echo JBaltnet:$opt >>$outfile
						opt=$(echo $line | cut -d: -f12) 	# exlist
						echo JBexlist:$opt >>$outfile
						opt=$(echo $line | cut -d: -f13) 	# snap
						echo JBsnap:$opt >>$outfile
						opt=$(echo $line | cut -d: -f14) 	# mediaopt
						echo JBbootmedia:$opt >>$outfile
						opt=$(echo $line | cut -d: -f15) 	# verify
						echo JBverify:$opt >>$outfile
						opt=$(echo $line | cut -d: -f16) 	# enckeyid
						echo JBenckeyid:$opt >>$outfile
						opt=$(echo $line | cut -d: -f17) 	# orascript
						echo JBorascript:$opt >>$outfile
					done < $datapath/joblist
				}
				mv $datapath/joblist $savedir 2>/dev/null

				##################
				# Job schedule exceptions
				##################
				for file in $(ls $datapath/jobs/*:except 2>/dev/null)
				do
					name=$(basename $file | cut -d: -f1)
					[ $name = GLOBAL ] && continue
					outfile=$datapath/jobs/$name:holidays
					echo JHjobid:$name >$outfile
					varlist="global first last day"
					for var in $varlist
					do
						val=$(grep ^$var: $file | cut -d: -f2)
						[ -z "$val" ] && val=0
						case $var in
							day) newvar=JHdate;;
							*) newvar=JH$var;;
						esac
						echo $newvar:$val >>$outfile
					done
					mv $file $savedir
				done

				##################
				# Boot media configuration files (cd and netboot only)
				##################
				for file in $(ls $datapath/media/* 2>/dev/null)
				do
					name=$(basename $file | cut -d: -f4)
					type=$(basename $file | cut -d: -f1)
					outfile=$datapath/bootmedia/$name:$type

					bdir=$(basename $file | cut -d: -f3)
					bsvr=$(basename $file | cut -d: -f2)
					[ $type = cd -a $bdir = local ] && bsvr="Store On Client"
					echo BMbootserver:$bsvr >$outfile
					echo BMmediatype:$type >>$outfile

					varlist="MAKECLIENT MAKEDATE DESC IMAGE DEVICE OSTYPE OSLEVEL PLATFORM NETTYPE INSTSERVER INSTALTNET INSTCLIENT APIBACKUPID APIFROMCLIENT APISERVER PROMPTMODE CONSOLE RIMSTART RIMLOGIN RIMDEV RIMIP RIMMASK RIMPWD RIMENTIF RIMENTCON RIMSPEED TERM RESTDEV BUSEQ VGOPT CLIENTIP NETDEV GATEWAY SUBMASK RINGSPEED ENTIF ENTCON"
					for var in $varlist
					do
						val=$(grep ^$var $file | cut -d" " -f2-99)
						[ ! -z "$val" -a $var = "$val" ] && {
							# Probably a line that does not have a value
							[ $(grep ^$var $file | wc -w) -eq 1 ] && val=
						}
						case $var in
							MAKECLIENT) newvar=BMmakeclient;;
							MAKEDATE) newvar=BMmakedate;;
							DESC) newvar=BMdesc;;
							IMAGE) newvar=BMname;;
							DEVICE) newvar=BMdevice;;
							OSTYPE) newvar=BMostype;;
							OSLEVEL) newvar=BMoslevel;;
							PLATFORM) newvar=BMplatform;;
							NETTYPE) newvar=BMnettype;;
							INSTSERVER) newvar=BMinstserver;;
							INSTALTNET) newvar=BMinstaltnet;;
							INSTCLIENT) newvar=BMinstclient;;
							APIBACKUPID) newvar=BMapibackupid;;
							APIFROMCLIENT) newvar=BMapifromclient;;
							APISERVER) newvar=BMapiserver;;
							PROMPTMODE) newvar=BMprompt;;
							CONSOLE) newvar=BMconsole;;
							RIMSTART) newvar=BMrimstart;;
							RIMLOGIN) newvar=BMrimlogin;;
							RIMDEV) newvar=BMrimdev;;
							RIMIP) newvar=BMrimip;;
							RIMMASK) newvar=BMrimmask;;
							RIMPWD) newvar=BMrimpwd;;
							RIMENTIF) newvar=BMrimentif;;
							RIMENTCON) newvar=BMrimentcon;;
							RIMSPEED) newvar=BMrimspeed;;
							TERM) newvar=BMterm;;
							RESTDEV) newvar=BMrestdev
									if [[ $val = /* ]]
									then
										basefile=$(basename $val)
										val=$(dirname $val)
										[[ $val = /images ]] && val=SBDIR
										echo BMbasefile:$basefile >>$outfile
									else
										echo BMbasefile: >>$outfile
									fi
								;;
							BUSEQ) newvar=BMbuseq;;
							VGOPT) newvar=BMvgopt;;
							CLIENTIP) newvar=BMclientip;;
							NETDEV) newvar=BMnetdev;;
							GATEWAY) newvar=BMgateway;;
							SUBMASK) newvar=BMsubmask;;
							RINGSPEED) newvar=BMringspeed;;
							ENTIF) newvar=BMentif;;
							ENTCON) newvar=BMentcon;;
							*) continue;;
						esac
						echo "$newvar:$val" >>$outfile
					done
					echo BMarch: >>$outfile
					mv $file $savedir 2>/dev/null
				done

				##################
				# Exclude lists
				##################
				for file in $(ls $datapath/exclude/* 2>/dev/null)
				do
					name=$(basename $file)
					outfile=$datapath/excludes/$name:info
					echo EXname:$name >$outfile
					opt=$(grep ^F: $file | cut -d: -f2-99)
					echo "EXfilelist:$opt" >>$outfile
					opt=$(grep ^D: $file | cut -d: -f2-99)
					echo "EXdevlist:$opt" >>$outfile
					opt=$(grep ^C: $file | cut -d: -f2-99)
					echo EXclients:$opt >>$outfile
					mv $file $savedir 2>/dev/null
				done
				rm -rf $datapath/exclude 2>/dev/null

				##################
				# Global holidays (job exceptions)
				##################
				[ -f $datapath/jobs/GLOBAL:except ] && {
					opt=$(grep ^day: $datapath/jobs/GLOBAL:except | cut -d: -f2)
					[ ! -z "$opt" ] && {
						echo HDdate:$opt >$datapath/holidays/holidays:info
					}
					mv $datapath/jobs/GLOBAL:except $savedir 2>/dev/null
				}

				##################
				# Printer (preferences)
				##################
				[ -s $datapath/printerlist ] && {
					while read line
					do
						name=$(echo $line | cut -d: -f1)
						outfile=$datapath/printers/$name:info
						echo PRname:$name >$outfile
						opt=$(echo $line | cut -d: -f2)
						echo PRcommand:$opt >>$outfile
						opt=$(echo $line | cut -d: -f3)
						echo PRdefault:$opt >>$outfile
					done < $datapath/printerlist
				}
				mv $datapath/printerlist $savedir 2>/dev/null

				##################
				# Backup label files
				##################
				echo "    Migrating backup label files ..."
				cd $datapath/labs
				for file in $(ls *:*:*:*^* 2>/dev/null)
				do
					newfile=$(echo $file | sed "s/\^/~/g")
					dev=$(echo $file | cut -d: -f3)
					[ $dev = disk ] && {
						svr=$(echo $file | cut -d: -f2)
						dev=$(echo $file | cut -d: -f4| sed "s/\^/~/g")
						dev=$(chkDevName $dev)
						newfile=svr:$svr:$dev
					}
					mv $file $newfile
				done

				##################
				# Backup status files
				##################
				# Remove status, cmdout, and errout files for backups whose
				# labels no longer exist.
				echo "    Migrating backup status files ..."
				cd $datapath/status
				for file in $(ls 2>/dev/null)
				do
					bid=$(echo $file | cut -d: -f4)
					[ ! -f $datapath/labs/lab:$bid ] && rm -f $file
				done

				cd $datapath/cmdout
				for file in $(ls 2>/dev/null)
				do
					bid=$(echo $file | cut -d: -f4)
					[ ! -f $datapath/labs/lab:$bid ] && rm -f $file
				done

				cd $datapath/errout
				for file in $(ls 2>/dev/null)
				do
					bid=$(echo $file | cut -d: -f4)
					[ ! -f $datapath/labs/lab:$bid ] && rm -f $file
				done
				cd /

			# Next group
			done
			export STX_GROUP=$oldgrp
			export STX_GUID=$oldgrpid
		}
	}

	[ $VRM -eq 720 ] && {
		[ $license_type = w ] && {
			# Sparse option may have been saved as a 1 or 0 using webif.
			# Option should be Y or N. This was fixed at AIX v7.2.0.8,
			# Linux/Solaris v7.2.0.7
			file=$STXPATH/main/data/clients/local:info
			opt=$(grep ^CLsparse $file 2>/dev/null | cut -d: -f2)
			case "$opt" in
				0) opt=N ;;
				1) opt=Y ;;
				*) opt= ;;
			esac
			[ ! -z "$opt" ] && {
				grep -v ^CLsparse $file >/tmp/stxinfo
				echo CLsparse:$opt >>/tmp/stxinfo
				mv /tmp/stxinfo $file
			}
		}
	}

	[ $VRM -lt 810 ] && {
		[ $license_type != c ] && {
			# New Storix theme created. Apply the new theme.
			# Do this for all groups.
			# GUI theme kept here
			for file in $(ls $STXPATH/*/config/display_attrs 2>/dev/null)
			do
				# Move the file out of the way so that the default
				# Storix theme will be used.
				mv $file $file.$VRM
			done
			for file in $(ls $STXPATH/*/config/app_settings 2>/dev/null)
			do
				# Remove the THEME line so that the default Storix theme
				# will be used.
				grep -v ^THEME: $file >/tmp/stx_appsettings
				if [ -s /tmp/stx_appsettings ]
				then
					mv $file $file.$VRM
					mv /tmp/stx_appsettings $file
				else
					rm -f /tmp/stx_appsettings
				fi
			done
		}
	}
	[ $VRM -lt 820 ] && {
		# Beginning with 8.2.0 SBDIR is no longer defined in 
		# /etc/fstab,/etc/filesystems or /etc/vfstab. We need to unmount
		# and remove any entries in fstab. The disk will be imported/updated
		# as backup jobs are run, or is manually imported. Do this on any client
		# admin or workstation system.

		file="$STXPATH/config/vdev:SBDIR"
		[ -f $file ] && {
			mountpoint=$(grep ^DVdevlist $file|cut -f2 -d:)
			[ ! -z $mountpoint ] && {

				# Clean fstab, or equivalent.
				[ "$LOCAL_OSTYPE" = "Linux" ] && {
					# If the mountpoint is mounted, unmount.
					grep -w $mountpoint /proc/mounts >/dev/null 2>&1 && {
						umount $mountpoint >/dev/null 2>&1
						[ $? -ne 0 ] && {
							$PRINT "WARNING: Unable to unmount $mountpoint."
							$PRINT "Please manually unmount the directory."
						}
					}
					[ -f /etc/fstab ] && {
						# We're on linux, remove from fstab.
						grep -v "$mountpoint" /etc/fstab >$STXTEMP/$$tmp-fstab
						mv $STXTEMP/$$tmp-fstab /etc/fstab
					}
				}
				[ "$LOCAL_OSTYPE" = "Solaris" ] && {
					grep "[	]$mountpoint[	]"  /etc/mnttab >/dev/null 2>&1 && {
						umount $mountpoint >/dev/null 2>&1
						[ $? -ne 0 ] && {
							$PRINT "WARNING: Unable to unmount $mountpoint."
							$PRINT "Please manually unmount the directory."
						}
					}
					[ -f /etc/vfstab ] && {
						# We're on Solaris, remove from vfstab.
						grep -v "[ ]$mountpoint[ ]" /etc/vfstab >\
							$STXTEMP/$$tmp-vfstab
						mv $STXTEMP/$$tmp-vfstab /etc/vfstab
					}
				}
				[ "$LOCAL_OSTYPE" = "AIX" ] && {
					mount 2>/dev/null |grep -w $mountpoint >/dev/null && {
						umount $mountpoint >/dev/null 2>&1
						[ $? -ne 0 ] && {
							$PRINT "WARNING: Unable to unmount $mountpoint."
							$PRINT "Please manually unmount the directory."
						}
					}
					[ -f /etc/filesystems ] && {
						# on AIX just export the vg and it will be removed.
						err=$(varyoffvg stbackupvg)
						[ $? -ne 0 ] && {
							$PRINT "WARNING: Unable to varyoffvg stbackupvg."
							$PRINT "Please run the command \"varyoffvg stbackupvg\" manually."
						}
						exportvg stbackupvg
						[ $? -ne 0 ] && {
							$PRINT "WARNING: Unable to exportvg stbackupvg."
							$PRINT "Please run the command \"exportvg stbackupvg\" manually."
						}
					}
				}
			}
		}
	}

	[ $unsched_jobs = 1 ] && {
		# Migrating to TSM edition. We must unschedule any currently
		# scheduled jobs.
		joblist=$(/usr/bin/stjob -l 2>/dev/null)
		[ ! -z "$joblist" ] && {
			$PRINT "\07$SEPARATOR
      Because you have converted your license to TSM Edition, it is
      necessary to unschedule all previously configured backup jobs.
      Should you decide to revert back to the original license type,
      the backup jobs will need to be rescheduled."
			$PRINT "$SEPARATOR"
			$PRINT "Unscheduling backup jobs ..."
			for job in $joblist
			do
				/usr/bin/stjob -a -wo $job >/dev/null 2>&1
			done
			sleep 5
		}
	}
}
